/**
 * @typedef {"html" | "css" | "js"} SupportedLanguages
 * @typedef {HTMLTextAreaElement | HTMLScriptElement | HTMLTemplateElement} SafeTemplateElements
 */
/**
 * A lightweight codepen-like editor. `<light-pen>` is `~15kb` in size. Compared to `~90kb` from CodeMirror and `~160kb` from CodeSandbox. LightPen purposely is small and lightweight and is not intended to replace these tools. But rather
 fill a gap for users who don't need as much power.
 * @customElement
 * @tagname light-pen
 * @slot html - HTML to insert
 * @slot css - CSS to insert
 * @slot js - JavaScript to insert
 * @slot title - The title to appear at the top of the editor
 *
 * @part base - The base wrapper
 * @part sandbox - The wrapper around the editor and the iframe
 * @part sandbox-header - The wrapper around the header area
 */
export default class LightPen extends BaseElement {
    /**
     * @override
     */
    static override styles: import("lit").CSSResult[];
    /**
     * @override
     */
    static override dependencies: {
        "light-editor": typeof LightEditor;
        "light-disclosure": typeof LightDisclosure;
    };
    /**
     * @override
     */
    static override properties: {
        openLanguages: {
            reflect: boolean;
            attribute: string;
        };
        resizePosition: {
            attribute: string;
            reflect: boolean;
            type: NumberConstructor;
        };
        preserveWhitespace: {
            attribute: string;
            type: BooleanConstructor;
            reflect: boolean;
        };
        console: {
            reflect: boolean;
        };
        sandboxSettings: {
            reflect: boolean;
            attribute: string;
        };
        languages: {
            attribute: boolean;
            type: ArrayConstructor;
        };
        cssCode: {
            attribute: boolean;
        };
        htmlCode: {
            attribute: boolean;
        };
        jsCode: {
            attribute: boolean;
        };
        htmlResizeObserver: {
            attribute: boolean;
        };
        jsResizeObserver: {
            attribute: boolean;
        };
        cssResizeObserver: {
            attribute: boolean;
        };
        _resizing: {
            attribute: boolean;
        };
    };
    /**
     * @property
     * @type {ResizeObserver}
     */
    resizeObserver: ResizeObserver;
    /**
     * @attribute
     * @reflect
     * @type {number}
     */
    resizePosition: number;
    /**
     * Languages to have open on initial render
     * Comma separated list of elements to open on initial render "html,css,js" to open all.
     * @attribute
     * @reflect
     * @type {string}
     */
    openLanguages: string;
    /**
     * @type {Array<SupportedLanguages>}
     * @property
     */
    languages: Array<SupportedLanguages>;
    /**
     * Not implemented.
     * @property
     * @type {"enabled" | "disabled"}
     */
    console: "enabled" | "disabled";
    /**
     * Not implemented.
     * @property
     * @type {string}
     */
    consoleText: string;
    /**
     * @property
     * @internal
     * @type {number}
     */
    cachedWidth: number;
    /**
     * @property
     * @type {string}
     */
    sandboxSettings: string;
    /**
     * @internal
     */
    _resizing: boolean;
    cssCode: string;
    htmlCode: string;
    jsCode: string;
    /**
     * Passed down to the editors.
     * @type {boolean}
     */
    preserveWhitespace: boolean;
    /**
     * @type {undefined | null | LightEditor}
     */
    get jsEditor(): LightEditor | null | undefined;
    /**
     * @type {undefined | null | LightEditor}
     */
    get cssEditor(): LightEditor | null | undefined;
    /**
     * @type {undefined | null | LightEditor}
     */
    get htmlEditor(): LightEditor | null | undefined;
    /**
     * Sets an initial width so we dont need to keep computing getBoundingClientRect
     */
    updateCachedWidth(): void;
    /**
     * @param {ResizeObserverEntry[]} entries
     */
    handleResize(entries: ResizeObserverEntry[]): void;
    /** Getters / Setters */
    /**
     * @returns {null | undefined | HTMLIFrameElement}
     */
    get iframeElem(): HTMLIFrameElement | null | undefined;
    updateIframeContent(): void;
    _iframeDebounce: NodeJS.Timeout | undefined;
    /**
     * @param {KeyboardEvent} event
     */
    handleResizerKeydown(event: KeyboardEvent): void;
    updateResizePosition(resizePosition?: number): void;
    resetValues(): void;
    renderConsole(): import("lit").TemplateResult<1>;
    /**
     * @override
     */
    override render(): import("lit").TemplateResult<1>;
    /**
     * @param {PointerEvent} event
     */
    handleDrag(event: PointerEvent): void;
    /**
     * @internal
     * @param {number} pixels
     * @return {number}
     */
    pixelsToPercentage(pixels: number): number;
    /**
     * @internal
     * @param {SupportedLanguages} language
     */
    renderEditor(language: SupportedLanguages): import("lit").TemplateResult<1>;
    /**
     * @internal
     * @param {SupportedLanguages} language
     */
    renderDetails(language: SupportedLanguages): import("lit").TemplateResult<1>;
}
export type SupportedLanguages = "html" | "css" | "js";
export type SafeTemplateElements = HTMLTextAreaElement | HTMLScriptElement | HTMLTemplateElement;
import { BaseElement } from "../../../internal/base-element.js";
import LightEditor from "../light-editor/light-editor.js";
import LightDisclosure from "../light-disclosure/light-disclosure.js";
